/**
 * Copyright (C) 2009-2012 enStratus Networks Inc
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.google;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Map;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;
import org.apache.log4j.Logger;
import org.dasein.cloud.AbstractCloud;
import org.dasein.cloud.CloudProvider;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.dc.DataCenterServices;
import org.dasein.cloud.google.storage.GoogleStorageServices;

public class GoogleAppEngine extends AbstractCloud {
    static private final Logger logger = Logger.getLogger(GoogleAppEngine.class);
    
    static public final String P_CFAUTH            = "Authorization";
    static public final String P_DATE              = "x-goog-date";
    
    static public String encode(String value, boolean encodePath) throws InternalException {
        String encoded = null;
        
        try {
            encoded = URLEncoder.encode(value, "utf-8").replace("+", "%20").replace("*", "%2A").replace("%7E","~");
            if( encodePath ) {
                encoded = encoded.replace("%2F", "/");
            }
        } 
        catch( UnsupportedEncodingException e ) {
            e.printStackTrace();
            throw new InternalException(e);
        }
        return encoded;     
    }
    
    public GoogleAppEngine() { }
    
    @Override
    public DataCenterServices getDataCenterServices() {
        CloudProvider compute = getComputeCloud();
        
        return (compute == null ? new GoogleDataCenter(this) : compute.getDataCenterServices());
    }
    
    @Override
    public GoogleStorageServices getStorageServices() {
        return new GoogleStorageServices(this);
    }
    
    @Override
    public String getCloudName() {
        return "Google AppEngine";
    }

    @Override
    public String getProviderName() {
        return "Google";
    }

    private String sign(byte[] key, String authString, String algorithm) throws InternalException {
        try {
            Mac mac = Mac.getInstance(algorithm);
            
            mac.init(new SecretKeySpec(key, algorithm));
            return new String(Base64.encodeBase64(mac.doFinal(authString.getBytes("utf-8"))));
        } 
        catch( NoSuchAlgorithmException e ) {
            e.printStackTrace();
            throw new InternalException(e);
        } 
        catch( InvalidKeyException e ) {
            e.printStackTrace();
            throw new InternalException(e);
        } 
        catch( IllegalStateException e ) {
            e.printStackTrace();
            throw new InternalException(e);
        } 
        catch( UnsupportedEncodingException e ) {
            e.printStackTrace();
            throw new InternalException(e);
        }
    }
    
    public String signForStorage(String accessKey, byte[] secretKey, String action, String hash, String contentType, Map<String,String> headers, String bucket, String object) throws InternalException {
        StringBuilder toSign = new StringBuilder();
        
        toSign.append(action);
        toSign.append("\n");
        if( hash != null ) {
            toSign.append(hash);
        }
        toSign.append("\n");
        if( contentType != null ) {
            toSign.append(contentType);
        }
        toSign.append("\n\n");
        ArrayList<String> keys = new ArrayList<String>();
        keys.addAll(headers.keySet());
        Collections.sort(keys);
        for( String hkey : keys ) {
            if( hkey.startsWith("x-amz") || hkey.startsWith("x-goog")) {
                String val = headers.get(hkey);
                
                if( val != null ) {
                    toSign.append(hkey);
                    toSign.append(":");
                    toSign.append(headers.get(hkey).trim());
                    toSign.append("\n");
                }
            }
        }
        toSign.append("/");
        if( bucket != null ) {
            toSign.append(bucket.toLowerCase());
            toSign.append("/");
        }
        if( object != null ) {
            toSign.append(object.toLowerCase());
        }
        String signature = sign(secretKey, toSign.toString(), "HmacSHA1");
        
        return ("GOOG1" + " " + accessKey + ":" + signature);
    }
    
    @Override
    public String testContext() {
        try {
            if( !getStorageServices().getBlobStoreSupport().isSubscribed() ) {
                return null;
            }
            return getContext().getAccountNumber();
        }
        catch( Throwable t ) {
            logger.warn("Failed to test Google connection context: " + t.getMessage());
            if( logger.isDebugEnabled() ) {
                t.printStackTrace();
            }
            return null;
        }
    }
}
